/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CompileRulePackageCommand.as 5084 2009-08-28 09:03:19Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.manager.command
{
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.utils.getQualifiedClassName;

import mx.collections.ArrayCollection;
import mx.rpc.IResponder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideException;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.event.*;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.manager.business.*;
import org.ow2.jasmine.monitoring.eos.rules.manager.event.*;
import org.ow2.jasmine.monitoring.eos.rules.manager.model.RuleManagerModelLocator;
import org.ow2.jasmine.monitoring.eos.rules.manager.model.domain.RuleManagerModel;

/**
* Compile the rule package
* 
* @author Julien Nicoulaud
*/
[Event(name="serverSideException", type="org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent")] 
public class CompileRulePackageCommand implements ICommand, IResponder
{

    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * The traited rule package
    */
    private var synchronizedRulePackage : SynchronizedRulePackage = null;
    
    
    // =========================================================================
    // Execution
    // =========================================================================
    
    /**
    * Send the event to the java side, using the business layer of the pattern
    */
    public function execute(event:CairngormEvent):void
    {
        // Retrieve the component model
        var model:RuleManagerModel = 
            RuleManagerModelLocator.getInstance().ruleManagerModel;
        
        // Retrieve the target object
        synchronizedRulePackage = (event as SynchronizedRulePackageEvent).rulePackage;
                            
        // Dispatch a notification to Kerneos
        CairngormEventDispatcher.getInstance().dispatchEvent(
            new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION + model.componentID,
                                         (getQualifiedClassName(this).split("::"))[1] + ": Compiling \"" +
                                         synchronizedRulePackage.rulePackage.displayName + "\"",
                                         KerneosNotificationEvent.DEBUG)
        );
                
        // Retrieve the command delegate
        var delegate:ICompileRulePackageDelegate = 
        	RuleManagerModelLocator.getInstance().getCompileRulePackageDelegate();
        
        // Set this object as a responder
        delegate.responder = this;
        
        // Execute a server-side method thanks to the delegate
        delegate.compileRulePackage(synchronizedRulePackage.rulePackage,
                                    model.ruleEngineName);
    }


    // =========================================================================
    // Result handling
    // =========================================================================

    /**
    * Get the result of the java side. this method is called on each event
    * returned from the Java side.
    */
    public function result(event : Object):void
    {
        // Retrieve the errors collection
        var errors : ArrayCollection = (event as ResultEvent).result as ArrayCollection;
        
        // Update the rule package errors collection
        synchronizedRulePackage.rulePackage.compilationErrors = errors;
        
        // Mark the compilation as checked
        synchronizedRulePackage.rulePackage.compilationChecked = true;
    }


    // =========================================================================
    // Fault handling
    // =========================================================================
    
    /**
    * Handle fault messages
    */
    public function fault(event : Object) : void
    {
        // Retrieve the fault event
        var faultEvent : FaultEvent = FaultEvent(event);

        // Tell the view and let it handle this
        var serverSideExceptionEvent : ServerSideExceptionEvent =
            new ServerSideExceptionEvent(
                "serverSideException",
                new ServerSideException("Error while compiling the rule package",
                                        "The operation could not be performed."
                                        + "\n" + faultEvent.fault.faultCode
                                        + "\n" + faultEvent.fault.faultString,
                                        faultEvent.fault.getStackTrace()));
        CairngormEventDispatcher.getInstance().dispatchEvent(serverSideExceptionEvent);
    }

}
}
