/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SaveRulePackageCommand.as 4860 2009-08-10 13:59:20Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.manager.command
{
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.utils.getQualifiedClassName;

import mx.collections.ArrayCollection;
import mx.rpc.IResponder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideException;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.event.RulePackageSystemNameEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.RulePackageSaveReportVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.AbstractRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.manager.business.*;
import org.ow2.jasmine.monitoring.eos.rules.manager.event.*;
import org.ow2.jasmine.monitoring.eos.rules.manager.model.RuleManagerModelLocator;
import org.ow2.jasmine.monitoring.eos.rules.manager.model.domain.RuleManagerModel;

/**
* Save a given rule package
* 
* @author Julien Nicoulaud
*/
[Event(name="serverSideException", type="org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent")] 
[Event(name="rulePackageModified", type="org.ow2.jasmine.monitoring.eos.rules.common.event.RulePackageSystemNameEvent")]
public class SaveRulePackageCommand implements ICommand, IResponder
{

    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * The traited rule package
    */
    private var synchronizedRulePackage : SynchronizedRulePackage = null;
    
    
    // =========================================================================
    // Execution
    // =========================================================================
    
    /**
    * Send the event to the java side, using the business layer of the pattern
    */
    public function execute(event:CairngormEvent):void
    {
        // Retrieve the command delegate
        var delegate:ISaveRulePackageDelegate = 
        	RuleManagerModelLocator.getInstance().getSaveRulePackageDelegate();
        
        // Set this object as a responder
        delegate.responder = this;
        
        // Store the target rule package
        synchronizedRulePackage = (event as SynchronizedRulePackageEvent).rulePackage;
                
        // Retrieve the component model
        var model:RuleManagerModel = 
            RuleManagerModelLocator.getInstance().ruleManagerModel;
                
        // Dispatch a notification to Kerneos
        CairngormEventDispatcher.getInstance().dispatchEvent(
            new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION + model.componentID,
                                         (getQualifiedClassName(this).split("::"))[1] + ": Saving \"" +
                                         synchronizedRulePackage.rulePackage.displayName + "\"",
                                         KerneosNotificationEvent.DEBUG)
        );
        
        // Execute a server-side method thanks to the delegate
        delegate.saveRulePackage(synchronizedRulePackage.rulePackage,
                                 model.ruleEngineName);
    }


    // =========================================================================
    // Result handling
    // =========================================================================

    /**
    * Get the result of the java side. this method is called on each event
    * returned from the Java side.
    */
    public function result(event : Object):void
    {
        // Retrieve the component model
        var model:RuleManagerModel = 
            RuleManagerModelLocator.getInstance().ruleManagerModel;
        
        // Mark the operation as comppleted
        model.pendingSaveOperations--;
            
    	// Get the server response
    	var serverResponse : RulePackageSaveReportVO =
    	   (event as ResultEvent).result as RulePackageSaveReportVO;
    	
    	// Retrieve failures and notify the view
    	for each (var failure : ArrayCollection in serverResponse.failures as ArrayCollection) {
    		
    		// Build a server side exception object
    		var serverSideException : ServerSideException = 
    		    new ServerSideException(failure[0],failure[1]);
    		
    		// If stacktrace provided, add it
    		if (failure.length>2) {
    			serverSideException.stackTrace = failure[2];
    		}
    		
    		// Build an event and dispatch it
            var serverSideExceptionEvent : ServerSideExceptionEvent =
                new ServerSideExceptionEvent(ServerSideExceptionEvent.SERVER_SIDE_EXCEPTION,
                                             serverSideException);
            CairngormEventDispatcher.getInstance().dispatchEvent(serverSideExceptionEvent);
    	}
        
        // If the rule package was activated|deactivated
        if (serverResponse.successes.contains(RulePackageSaveReportVO.OPERATION_ACTIVATE)) {
            
            // Update the state
            synchronizedRulePackage.rulePackage.state = AbstractRulePackageVO.ACTIVATED;
            
        } else if (serverResponse.successes.contains(RulePackageSaveReportVO.OPERATION_DEACTIVATE)) {
            
            // Update the state
            synchronizedRulePackage.rulePackage.state = AbstractRulePackageVO.DEACTIVATED;
        }
        
        // If it was saved
        if (serverResponse.successes.contains(RulePackageSaveReportVO.OPERATION_SAVE)) {
            
            // Dispatch a notification event
            var changeEvent : RulePackageSystemNameEvent;
            if (synchronizedRulePackage.rulePackage.previousSystemName !== null) {
                changeEvent = new RulePackageSystemNameEvent(RulePackageSystemNameEvent.MANAGER_RULE_PACKAGE_MODIFIED + model.componentID,
                                                             synchronizedRulePackage.rulePackage.previousSystemName);
            } else {
                changeEvent = new RulePackageSystemNameEvent(RulePackageSystemNameEvent.MANAGER_RULE_PACKAGE_MODIFIED + model.componentID,
                                                             synchronizedRulePackage.rulePackage.systemName);
            }
            CairngormEventDispatcher.getInstance().dispatchEvent(changeEvent);
            
            // Update the opened rule package object
            synchronizedRulePackage.rulePackage.previousSystemName = null;
            
            // Update the state
            synchronizedRulePackage.state = SynchronizedRulePackageState.SYNCHRONIZED;
            
        }
    }


    // =========================================================================
    // Fault handling
    // =========================================================================
    
    /**
    * Handle fault messages
    */
    public function fault(event : Object) : void
    {
        // Retrieve the component model
        var model:RuleManagerModel = 
            RuleManagerModelLocator.getInstance().ruleManagerModel;
        
        // Mark the operation as comppleted
        model.pendingSaveOperations--;
    	
        // Retrieve the fault event
        var faultEvent : FaultEvent = FaultEvent(event);

        // Tell the view and let it handle this
        var serverSideExceptionEvent : ServerSideExceptionEvent =
            new ServerSideExceptionEvent(
                "serverSideException",
                new ServerSideException("Error while saving the rule package",
                                        "The operation could not be performed."
                                        + "\n" + faultEvent.fault.faultCode
                                        + "\n" + faultEvent.fault.faultString,
                                        faultEvent.fault.getStackTrace()));
        CairngormEventDispatcher.getInstance().dispatchEvent(serverSideExceptionEvent);
    }

}
}
